///////////////////////////////////////////////////////////////////////////////
//
//  Class code created by the Xbase++ FormDesigner
//    Creation date: 05.11.2007 Creation time: 00:10:23
//
//  Contents  :
//    This file contains the basic structure for the utilization-level of
//    a form. It may (and should) be modified.
//
///////////////////////////////////////////////////////////////////////////////

#include "Gra.ch"
#include "Xbp.ch"
#include "Common.ch"
#include "Appevent.ch"
#include "Font.ch"

#include "StyleView.ch"

#define MIN_SCALED_WIDTH   90
#define MIN_SCALED_HEIGHT  25

******************************************************************************
* This class is derived from the implementation-level class of the form.
* Instance variables are declared in the _NewForm class.
******************************************************************************
CLASS NewForm FROM _NewForm
   EXPORTED:
      VAR StyleInfo

      VAR CurrStyleIdx
      VAR CurrStyle

      METHOD Init()
      METHOD Create()

      METHOD ClearStyleInfo()

      METHOD DisplayStyles()
      METHOD StyleSelected()
      METHOD DrawStyleBackground()
ENDCLASS

******************************************************************************
* Initialize form
******************************************************************************
METHOD NewForm:init( oParent, oOwner, aPos, aSize, aPP, lVisible )

   * Execute method of the super class
   ::_NewForm:init( oParent, oOwner, aPos, aSize, aPP, lVisible )

   ::Title := WINDOW_TITLE

   ::tvStyles:ItemMarked := {|oItem| ::StyleSelected(oItem)}
   ::staBackground:Paint := {|aRect| ::DrawStyleBackground(aRect)}

   ::StyleInfo    := {}
   ::CurrStyleIdx := 0

RETURN self


******************************************************************************
* Request system resources
******************************************************************************
METHOD NewForm:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

   * Execute method of the super class
   ::_NewForm:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

   ::sleCaption:SetData( "Test Caption" )
   ::ClearStyleInfo()

   CenterControl( self )

RETURN self


METHOD NewForm:DisplayStyles( aClasses, aParts, aStates )

 LOCAL i, j
 LOCAL nIndex
 LOCAL cPart
 LOCAL nPartId
 LOCAL nLastType
 LOCAL oLastClass
 LOCAL oLastPart
 LOCAL oParent
 LOCAL oTmp
 LOCAL aCurrStates
 LOCAL cCaption
 LOCAL aDefStates := {{"NORMAL",1}, ;
                      {"HOT",2},    ;
                      {"PRESSED",3},;
                      {"DISABLED",2}}
   // Build the info tree
   //
   // Format is as follows:
   // { {<cClass1>,<cPart1>,<nPartId1>,<cState1>,<nStateId1>,<nNodeType1>},;
   //   (...)
   //   {<cClassX>,<cPartX>,<nPartIdX>,<cStateX>,<nStateIdX>,<nNodeTypeX>} }
   //
   // See "StyleView.CH" for the definition of "nNodeType",
   // which defines what fields are field in for a certain
   // node.
   //
   // The array has a flat layout that translates easily
   // to the id of a treeview item, for instance.
   FOR i:=1 TO Len(aClasses)
      AAdd( ::StyleInfo, {aClasses[i],,,,,CLASS_NODE} )

      nIndex := AScan( aParts, aClasses[i] )
      IF nIndex == 0
         LOOP
      ENDIF

      j := nIndex
      DO WHILE j<=Len(aParts) .AND. aParts[j][1] == aClasses[i]
         cPart   := aParts[j][2]
         nPartId := aParts[j][3]

         AAdd( ::StyleInfo, {aClasses[i],cPart,nPartId,,,PART_NODE} )
         nIndex := AScan( aStates, cPart )
         IF nIndex != 0
            // Remember current states in case next
            // part doesn't define any
            aCurrStates := aStates[nIndex][2]
         ENDIF
         IF aCurrStates == NIL
            AEval( aDefStates, {|x| AAdd(::StyleInfo,{aClasses[i],cPart,nPartId,x[1],x[2],STATE_NODE})} )
            j++
            LOOP
         ENDIF

         AEval( aCurrStates, {|x| AAdd(::StyleInfo,{aClasses[i],cPart,nPartId,x[1],x[2],STATE_NODE})} )

         j++
      ENDDO

      aCurrStates := NIL
   NEXT

   //
   // Populate tree view
   //
   nLastType := CLASS_NODE
   oParent := ::tvStyles:RootItem

   FOR i:=1 TO Len(::StyleInfo)
      DO CASE
         CASE ::StyleInfo[i][NODE_TYPE] == CLASS_NODE
            oParent    := ::tvStyles:RootItem
            oLastClass := oParent:AddItem( ::StyleInfo[i][CLASS_NAME] )
            oLastClass:SetData( i )
            nLastType := CLASS_NODE
         CASE ::StyleInfo[i][NODE_TYPE] == PART_NODE
            IF nLastType != PART_NODE
               oParent   := oLastClass
               nLastType := PART_NODE
            ENDIF
            cCaption  := ::StyleInfo[i][PART_NAME] + " [Part Id: " + LTrim(Str(::StyleInfo[i][PART_ID])) + "]"
            oLastPart := oParent:AddItem( cCaption )
            oLastPart:SetData( i )
         CASE ::StyleInfo[i][NODE_TYPE] == STATE_NODE
            IF nLastType != STATE_NODE
               oParent   := oLastPart
               nLastType := STATE_NODE
            ENDIF
            cCaption  := ::StyleInfo[i][STATE_NAME] + " [State Id:" + LTrim(Str(::StyleInfo[i][STATE_ID])) + "]"
            oTmp := oParent:AddItem( cCaption )
            oTmp:SetData( i )
      ENDCASE
   NEXT

RETURN


METHOD NewForm:ClearStyleInfo()

   ::staContentRect:SetCaption( "" )
   ::staBGRect:SetCaption( "" )
   ::staCaptionRect:SetCaption( "" )
   ::staMinSize:SetCaption( "" )
   ::staDefSize:SetCaption( "" )
   ::staContentAlignment:SetCaption( "" )
   ::staCaptionRect:SetCaption( "" )
   ::staTextColor:SetColorBG( XBPSYSCLR_DIALOGBACKGROUND )
   ::staTextColor:SetCaption( "" )
   ::staFillColor:SetColorBG( XBPSYSCLR_DIALOGBACKGROUND )
   ::staFillColor:SetCaption( "" )

RETURN


METHOD NewForm:StyleSelected( oItem )

 LOCAL nIndex := oItem:GetData()
 LOCAL nTmp
 LOCAL lTmp   := .F.
 LOCAL aSize
 LOCAL aRect
 LOCAL aMargins
 LOCAL nColor

   IF ::CurrStyle != NIL
      ::CurrStyle:Free()
      ::CurrStyle := NIL

      IF ::StyleInfo[nIndex][NODE_TYPE] == STATE_NODE
         ::ClearStyleInfo()
      ENDIF
   ENDIF

   IF ::StyleInfo[nIndex][NODE_TYPE] != STATE_NODE
      RETURN
   ENDIF

   ::CurrStyleIdx := nIndex
   ::SetTitle( WINDOW_TITLE + "   [Style: " + ::StyleInfo[::CurrStyleIdx][CLASS_NAME] + ", " + ;
               "Part: " + ::StyleInfo[::CurrStyleIdx][PART_NAME] + "]" )
   ::CurrStyle := VisualStyle():New( ::StyleInfo[::CurrStyleIdx][CLASS_NAME], ;
                                     ::StyleInfo[::CurrStyleIdx][PART_ID],    ;
                                     ::StyleInfo[::CurrStyleIdx][STATE_ID] )
   ::staBackground:InvalidateRect()
   IF ::CurrStyle:Handle == 0
      RETURN
   ENDIF

   ::staMinSize:SetCaption( Var2Char(::CurrStyle:MinPartSize) )
   ::staDefSize:SetCaption( Var2Char(::CurrStyle:DefPartSize) )

   nTmp := ::CurrStyle:GetContentAlignment( @lTmp )
   IF lTmp == .F.
      nTmp := -1
   ENDIF

   DO CASE
      CASE nTmp == XBPALIGN_LEFT
         ::staContentAlignment:SetCaption( "Left" )
      CASE nTmp == XBPALIGN_RIGHT
         ::staContentAlignment:SetCaption( "Right" )
      CASE nTmp == XBPALIGN_HCENTER
         ::staContentAlignment:SetCaption( "Center" )
      OTHERWISE
         ::staContentAlignment:SetCaption( "(unspecified)" )
   ENDCASE

   aSize := ::CurrStyle:DefPartSize
   IF aSize[1] != 0 .OR. aSize[2] != 0
      ::staDefSize:SetCaption( Var2Char({0,0,aSize[1],aSize[2]}) )
   ELSE
      aSize := ::CurrStyle:MinPartSize
      ::staDefSize:SetCaption( "(unspecified)" )
   ENDIF

   IF ::staScaleImage:GetData() == .T.
      aSize[1] := Max( aSize[1], MIN_SCALED_WIDTH )
      aSize[2] := Max( aSize[2], MIN_SCALED_HEIGHT )
   ENDIF

   ::staBGRect:SetCaption( Var2Char({0,0,aSize[1],aSize[2]}) )
   aRect    := ::CurrStyle:CalcContentRect( {0,0,aSize[1],aSize[2]} )
   ::staContentRect:SetCaption( Var2Char(aRect) )
   aMargins := ::CurrStyle:GetCaptionMargins( @lTmp )

   IF lTmp == .T.
      aRect[1] += aMargins[1]
      aRect[2] += aMargins[4]
      aRect[3] += aMargins[2]
      aRect[4] += aMargins[3]
      ::staCaptionRect:SetCaption( Var2Char(aRect) )
   ELSE
      ::staCaptionRect:SetCaption( "(Unspecified)" )
   ENDIF

   nColor := ::CurrStyle:GetTextColor( @lTmp )
   IF lTmp == .F.
      ::staTextColor:SetCaption( "(unspecified)" )
   ELSE
      ::staTextColor:SetColorBG( nColor )
   ENDIF
   nColor := ::CurrStyle:GetFillColor( @lTmp )
   IF lTmp == .F.
      ::staFillColor:SetCaption( "(unspecified)" )
   ELSE
      ::staFillColor:SetColorBG( nColor )
   ENDIF

RETURN


METHOD NewForm:DrawStyleBackground( aRect )

 LOCAL oPS
 LOCAL aSize

   UNUSED( aRect )

   IF ::CurrStyle == NIL
      RETURN
   ENDIF

   aSize := ::CurrStyle:DefPartSize
   IF aSize[1] == 0 .AND. aSize[2] == 0
      aSize := ::CurrStyle:MinPartSize
      aRect := ::CurrStyle:CalcBoundsRect( {10,10,aSize[1]+10,aSize[2]+10} )
   ENDIF
   IF ::staScaleImage:GetData() == .T.
      aSize[1] := Max( aSize[1], MIN_SCALED_WIDTH )
      aSize[2] := Max( aSize[2], MIN_SCALED_HEIGHT )
      aRect := ::CurrStyle:CalcContentRect( {10,10,aSize[1]+10,aSize[2]+10} )
   ENDIF

   oPS   := ::staBackground:LockPS()

   ::CurrStyle:DrawBackground( oPS, {10,10}, {aSize[1]+10,aSize[2]+10} )
   ::CurrStyle:DrawCaption( oPS, {aRect[1],aRect[2]}, {aRect[3],aRect[4]}, ;
                            ::sleCaption:GetData() )
   ::UnlockPS()

RETURN

//EOF
/////