//////////////////////////////////////////////////////////////////////
//
//  VSTYLES.PRG
//
//  Copyright:
//      Alaska Software, (c) 2004-2009. All rights reserved.         
//  
//  Contents:
//      Classes and functions for visual style-support
//   
//////////////////////////////////////////////////////////////////////

#include "appevent.ch"
#include "xbp.ch"
#include "dll.ch"
#include "common.ch"

#PRAGMA LIBRARY( "XPPUI1.LIB" )

//
// Defines and Definitions
//
#define TMT_CONTENTALIGNMENT  4006
#define TMT_HALIGN            4005
#define TMT_VALIGN            4007
#define TMT_FILLTYPE          4003
#define TMT_GLYPHTYPE         4012

#define TMT_SIZINGMARGINS     3601
#define TMT_CONTENTMARGINS    3602
#define TMT_CAPTIONMARGINS    3603

#define TMT_BORDERCOLOR       3801
#define TMT_FILLCOLOR         3802
#define TMT_TEXTCOLOR         3803

#define TMT_TEXT              3201

#define TMT_TRANSPARENT       2201

#define TMT_BGFILL            2205
#define TMT_BORDERONLY        2203
#define TMT_GLYPHONLY         2207
#define TMT_GLYPHTRANSPARENT  2206

#define TS_MIN                0
#define TS_TRUE               1
#define TS_DRAW               2

#define CA_LEFT               0
#define CA_CENTER             1
#define CA_RIGHT              2

#define HA_LEFT               0
#define HA_CENTER             1
#define HA_RIGHT              2

#define VA_LEFT               0
#define VA_CENTER             1
#define VA_RIGHT              2

#define GT_NONE               0
#define GT_IMAGEGLYPH         1
#define GT_FONTGLYPH          2


//////////////////////////////////////////////////////////////////////
// Declaration of class "VisualStyle"
//////////////////////////////////////////////////////////////////////
CLASS VisualStyle
 PROTECTED:
   VAR           PartId
   VAR           State
   VAR           ClassName

   VAR           OwnsHandle

 EXPORTED:
   VAR           Handle
   ACCESS METHOD GetHandle()                  VAR Handle

   METHOD        Init()
   METHOD        Free()
   METHOD        AssignTo()

   METHOD        SetStyle()
   METHOD        ResetStyle()

   METHOD        CalcPartSize()
   ACCESS METHOD GetDefPartSize()             VAR DefPartSize
   ACCESS METHOD GetMinPartSize()             VAR MinPartSize

   ACCESS METHOD GetContentAlignment()        VAR ContentAlignment

   METHOD        CalcBoundsRect()
   METHOD        CalcContentRect()

   METHOD        IsPartDefined()
   ACCESS METHOD GetIsBackgroundTransparent() VAR IsBackgroundTransparent
   ACCESS METHOD GetHasBackground()           VAR HasBackground
   ACCESS METHOD GetCaptionMargins()          VAR CaptionMargins
   ACCESS METHOD GetContentMargins()          VAR ContentMargins

   ACCESS METHOD GetTextColor()               VAR TextColor
   ACCESS METHOD GetFillColor()               VAR FillColor

   METHOD        GetEnum()
   METHOD        GetBool()
   METHOD        GetInt()
   METHOD        GetMargins()
   METHOD        GetColor()

   SYNC METHOD   DrawBackground()
   SYNC METHOD   DrawCaption()
ENDCLASS


//////////////////////////////////////////////////////////////////////
// Implementation of class "VisualStyle"
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:Init( cClass, nPart, nState )

   ::Handle     := 0
   ::OwnsHandle := .F.

   IF Empty(cClass) == .F.
      ::SetStyle( cClass, nPart, nState )
   ELSE
      ::PartId    := 1
      ::State     := XBP_STATE_NORMAL
      ::ClassName := ""
   ENDIF

RETURN self


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:Free()
   ::ResetStyle()
RETURN self


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:AssignTo( oObj )

   IF ValType(oObj) != "O" .OR. oObj:IsDerivedFrom(XbpWindow()) == .F.
      XbpException():RaiseParameterType( {oObj} )
   ENDIF

   oObj:StyleClass  := ::ClassName
   oObj:StylePart   := ::PartId

RETURN .T.


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:SetStyle( xClass, nPart, nState )
 LOCAL nType   := ValType( xClass )
 LOCAL lReturn := .T.

   DEFAULT nPart  TO 1
   DEFAULT nState TO XBP_STATE_NORMAL

   IF nType $ "OC" == .F.
       XbpException():RaiseParameterType( {xClass,nPart,nState} )
   ENDIF

   IF nType == "O"
      IF xClass:IsDerivedFrom(XbpWindow()) == .F.
          XbpException():RaiseParameterType( {xClass,nPart,nState} )
      ENDIF

      ::Handle     := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetWindowTheme", ;
                               xClass:GetHWND() )
      nPart        := xClass:StylePart
      xClass       := xClass:StyleClass
      ::OwnsHandle := .F.
      ::ClassName  := xClass

      lReturn      := (::Handle != 0)
   ELSE
      ::OwnsHandle := .T.
      ::ClassName  := xClass  
   ENDIF

   ::PartId := nPart
   ::State  := nState

RETURN lReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetHandle()
 LOCAL cTmp

   IF ::Handle == 0 .AND. ::OwnsHandle == .T.
      cTmp      := Str2Unicode( ::ClassName )
      ::Handle  := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpOpenThemeData",  0, cTmp )
   ENDIF

RETURN ::Handle


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:CalcPartSize( aRect )
 LOCAL nResult
 LOCAL pRect
 LOCAL pSize
 LOCAL aReturn := {0,0}

   IF ::Handle == 0
      RETURN .F.
   ENDIF

   pRect   := U2Bin(0) + U2Bin(0) + U2Bin(aRect[3]-aRect[1]) + L2Bin(aRect[4]-aRect[2])
   pSize   := U2Bin(0) + U2Bin(0)
   nResult := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemePartSize",  ;
                       ::Handle, 0, ::PartId, ::State, pRect, TS_TRUE,   ;
                        @pSize )
   IF nResult == 0
      aReturn[1] := Bin2L( pSize )
      aReturn[2] := Bin2L( SubStr(pSize,5) )
   ENDIF

RETURN aReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetDefPartSize()
 LOCAL nResult
 LOCAL pSize
 LOCAL aReturn := {0,0}

   IF ::Handle == 0
      RETURN aReturn
   ENDIF

   pSize   := U2Bin(0) + U2Bin(0)
   nResult := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemePartSize",  ;
                       ::Handle, 0, ::PartId, ::State, 0, TS_DRAW, @pSize )
   IF nResult == 0
      aReturn[1] := Bin2L( pSize )
      aReturn[2] := Bin2L( SubStr(pSize,5) )
   ENDIF

RETURN aReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetMinPartSize()
 LOCAL nResult
 LOCAL pSize
 LOCAL aReturn := {0,0}

   IF ::Handle == 0
      RETURN aReturn
   ENDIF

   pSize   := U2Bin(0) + U2Bin(0)
   nResult := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemePartSize",  ;
                       ::Handle, 0, ::PartId, ::State, 0, TS_MIN, @pSize )
   IF nResult == 0
      aReturn[1] := Bin2L( pSize )
      aReturn[2] := Bin2L( SubStr(pSize,5) )
   ENDIF

RETURN aReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:CalcBoundsRect( aContentRect )
 LOCAL nResult
 LOCAL pRect
 LOCAL pContent
 LOCAL aReturn := {0,0,0,0}

   IF ::Handle == 0
      RETURN aReturn
   ENDIF

   pContent:= U2Bin(0) + U2Bin(0) + L2Bin(aContentRect[3]-aContentRect[1]) + ;
              L2Bin(aContentRect[4]-aContentRect[2])
   pRect   := U2Bin(0) + U2Bin(0) + U2Bin(0) + U2Bin(0)
   nResult := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemeBackgroundExtent",  ;
                       ::Handle, 0, ::PartId, ::State, pContent, @pRect )
   IF nResult == 0
      aReturn[1] := aContentRect[1] + Bin2L( pRect )
      aReturn[2] := aContentRect[2] + ((aContentRect[4]-aContentRect[2]) - ;
                    Bin2L( SubStr(pRect,13) ) )
      aReturn[3] := aContentRect[3] - ((aContentRect[3]-aContentRect[1]) - ;
                    Bin2L( SubStr(pRect,9) ) )
      aReturn[4] := aContentRect[4] - Bin2L( SubStr(pRect,5) )
   ENDIF

RETURN aReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:CalcContentRect( aBGRect )
 LOCAL nResult
 LOCAL pRect
 LOCAL pBounds
 LOCAL aSize
 LOCAL aReturn := {0,0,0,0}

   IF ::Handle == 0
      RETURN aReturn
   ENDIF

   aSize   := {aBGRect[3]-aBGRect[1],aBGRect[4]-aBGRect[2]}
   pBounds := U2Bin(0) + U2Bin(0) + L2Bin(aSize[1]) + L2Bin(aSize[2])
   pRect   := U2Bin(0) + U2Bin(0) + U2Bin(0) + U2Bin(0)
   nResult := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemeBackgroundContentRect",  ;
                       ::Handle, 0, ::PartId, ::State, pBounds, @pRect )
   IF nResult == 0
      aReturn[1] := aBGRect[1] + Bin2L( pRect )
      aReturn[2] := aBGRect[2] + (aSize[2] - Bin2L( SubStr(pRect,13) ) )
      aReturn[3] := aBGRect[3] - (aSize[1] - Bin2L( SubStr(pRect,9) ) )
      aReturn[4] := aBGRect[4] - Bin2L( SubStr(pRect,5) )
   ENDIF

RETURN aReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetContentAlignment( lIsDefined )
 LOCAL nReturn := XBPALIGN_LEFT
 LOCAL nAlign

   nAlign := ::GetEnum( TMT_CONTENTALIGNMENT, @lIsDefined )
   DO CASE
      CASE nAlign == CA_RIGHT
         nReturn := XBPALIGN_RIGHT
      CASE nAlign == CA_CENTER
         nReturn := XBPALIGN_HCENTER
   ENDCASE

RETURN nReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetEnum( nEnum, lIsDefined )
 LOCAL nReturn := 0
 LOCAL nError

   nError := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemeEnumValue", ;
                      ::Handle, ::PartId, ::State, nEnum, @nReturn )
   lIsDefined := (nError == 0)

RETURN nReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetBool( nBool, lIsDefined )
 LOCAL nReturn := 0
 LOCAL nError

   nError := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemeBoolValue", ;
                      ::Handle, ::PartId, ::State, nBool, @nReturn )
   lIsDefined := (nError == 0)

RETURN (nReturn!=0)


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetInt( nInt, lIsDefined )
 LOCAL nReturn := 0
 LOCAL nError

   nError := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemeIntValue", ;
                      ::Handle, ::PartId, ::State, nInt, @nReturn )
   lIsDefined := (nError == 0)

RETURN nReturn


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetMargins( nMargins, lIsDefined )
 LOCAL aMargins := {0,0,0,0}
 LOCAL pMargins
 LOCAL nError

    pMargins := U2Bin(0) + U2Bin(0) + U2Bin(0) + U2Bin(0)
    nError := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemeMargins", ;
                       ::Handle, 0, ::PartId, ::State, nMargins, 0, @pMargins )
    lIsDefined := (nError == 0)
    IF nError == 0
       aMargins[1] := Bin2L( pMargins )
       aMargins[2] := Bin2L( SubStr(pMargins, 5) )
       aMargins[3] := Bin2L( SubStr(pMargins, 9) )
       aMargins[4] := Bin2L( SubStr(pMargins, 13) )
    ENDIF

RETURN aMargins


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetColor( nColor, lIsDefined )
 LOCAL nError
 LOCAL nValue := 0

   nError := DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpGetThemeColor", ;
                       ::Handle, ::PartId, ::State, nColor, @nValue )
   lIsDefined := (nError == 0)
   IF lIsDefined == .T.
       nValue += 16777216
   ENDIF

RETURN nValue


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:IsPartDefined( nPart )
RETURN (DllCall("XPPUI1.DLL",DLL_CDECL,"_xbpIsThemePartDefined",::Handle,nPart,::State)!=0)


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetIsBackgroundTransparent()
RETURN ::GetBool(TMT_TRANSPARENT)


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetHasBackground()
RETURN !::GetBool(TMT_GLYPHONLY)


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetCaptionMargins( lIsDefined )
RETURN ::GetMargins(TMT_CAPTIONMARGINS, @lIsDefined)


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetContentMargins( lIsDefined )
RETURN ::GetMargins(TMT_CONTENTMARGINS, @lIsDefined)


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetTextColor( lIsDefined )
 LOCAL nColor

   nColor := ::GetColor( TMT_TEXTCOLOR, @lIsDefined )
   IF lIsDefined == .F.
      nColor := XBPSYSCLR_WINDOWTEXT
   ENDIF

RETURN nColor


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:GetFillColor( lIsDefined )
 LOCAL nColor

   nColor := ::GetColor( TMT_FILLCOLOR, @lIsDefined )
   IF lIsDefined == .F.
      nColor := XBPSYSCLR_DIALOGBACKGROUND
   ENDIF

RETURN nColor


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:ResetStyle()

   IF ::Handle == 0
      RETURN .T.
   ENDIF

   IF ::OwnsHandle == .T.
      DllCall( "XPPUI1.DLL", DLL_CDECL, "_xbpCloseThemeData", ::Handle )
   ENDIF

   ::Handle := 0

RETURN .T.


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:DrawBackground( oPS, aStart, aEnd, lErase, nState )
 LOCAL hOld
 LOCAL nStateOld

   DEFAULT nState TO ::State

   IF ::Handle == 0
      RETURN .F.
   ENDIF

   hOld := oPS:ThemeHandle
   oPS:ThemeHandle := ::Handle
   IF oPS:Window != NIL
      nStateOld := oPS:Window:ControlState
      oPS:Window:ControlState := nState
   ENDIF

   GraBackground( oPS, aStart, aEnd, lErase, ::PartId )
   oPS:ThemeHandle := hOld
   IF oPS:Window != NIL
      oPS:Window:ControlState := nStateOld
   ENDIF

RETURN .T.


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
METHOD VisualStyle:DrawCaption( oPS, aStart, aEnd, cCaption, nAlign, nState )
 LOCAL hOld
 LOCAL nStateOld

   DEFAULT nState TO ::State

   IF ::Handle == 0
      RETURN .F.
   ENDIF

   IF Empty(nAlign) == .T.
      nAlign := ::ContentAlignment + XBPALIGN_VCENTER
   ENDIF

   hOld := oPS:ThemeHandle
   oPS:ThemeHandle := ::Handle
   IF oPS:Window != NIL
      nStateOld := oPS:Window:ControlState
      oPS:Window:ControlState := nState
   ENDIF

   GraCaptionStr( oPS, aStart, aEnd, cCaption, nAlign,, ::PartId )
   oPS:ThemeHandle := hOld
   IF oPS:Window != NIL
      oPS:Window:ControlState := nStateOld
   ENDIF

RETURN .T.

// EOF
